import React from 'react';
import ReactDOM from 'react-dom';
import TestUtils from 'react-addons-test-utils';
import purebem from 'purebem';

import Checkbox from './Checkbox';

import simulateLabelClick from '../../test-utils/simulateLabelClick';


const TestCheckbox = Checkbox({
    block: purebem.of('test-checkbox'),
    displayName: 'TestCheckbox'
});

describe('Checkbox', () => {

    it('should render <label> as root', () => {
        // make sure we're using a label or our assumption when using 'simulateLabelClick' is false.
        const checkbox = TestUtils.renderIntoDocument(
            <TestCheckbox />
        );

        const node = ReactDOM.findDOMNode(checkbox);
        expect(node.tagName.toLowerCase()).toBe('label');
    });

    it('should be checked when clicked and is unchecked', () => {
        const onChange = jest.fn();

        const checkbox = TestUtils.renderIntoDocument(
            <TestCheckbox onChange={ onChange } value={ false } />
        );

        simulateLabelClick(checkbox);

        expect(onChange).toHaveBeenCalledWith(true);
    });

    it('should be unchecked when clicked and is checked', () => {
        const onChange = jest.fn();

        const checkbox = TestUtils.renderIntoDocument(
            <TestCheckbox onChange={ onChange } value={ true } />
        );

        simulateLabelClick(checkbox);

        expect(onChange).toHaveBeenCalledWith(false);
    });

    it('should remain unchecked when disabled and clicked', () => {
        const onChange = jest.fn();

        const checkbox = TestUtils.renderIntoDocument(
            <TestCheckbox disabled={ true } onChange={ onChange } value={ false } />
        );

        simulateLabelClick(checkbox);

        expect(onChange).not.toHaveBeenCalled();
    });

    it('should remain checked when disabled and clicked', () => {
        const onChange = jest.fn();

        const checkbox = TestUtils.renderIntoDocument(
            <TestCheckbox disabled={ true } onChange={ onChange } value={ true } />
        );

        simulateLabelClick(checkbox);

        expect(onChange).not.toHaveBeenCalled();
    });

    it('should be checked when using space and is unchecked', () => {
        const onChange = jest.fn();

        const checkbox = TestUtils.renderIntoDocument(
            <TestCheckbox onChange={ onChange } value={ false } />
        );

        const node = ReactDOM.findDOMNode(checkbox);
        TestUtils.Simulate.keyDown(node, { which: 32 });

        expect(onChange).toHaveBeenCalledWith(true);
    });

    it('should be unchecked when using space and is checked', () => {
        const onChange = jest.fn();

        const checkbox = TestUtils.renderIntoDocument(
            <TestCheckbox onChange={ onChange } value={ true } />
        );

        const node = ReactDOM.findDOMNode(checkbox);
        TestUtils.Simulate.keyDown(node, { which: 32 });

        expect(onChange).toHaveBeenCalledWith(false);
    });

    it('should remain unchecked when disabled and using space', () => {
        const onChange = jest.fn();

        const checkbox = TestUtils.renderIntoDocument(
            <TestCheckbox disabled={ true } onChange={ onChange } value={ false } />
        );

        const node = ReactDOM.findDOMNode(checkbox);
        TestUtils.Simulate.keyDown(node, { which: 32 });

        expect(onChange).not.toHaveBeenCalled();
    });

    it('should remain checked when disabled and using space', () => {
        const onChange = jest.fn();

        const checkbox = TestUtils.renderIntoDocument(
            <TestCheckbox disabled={ true } onChange={ onChange } value={ true } />
        );

        const node = ReactDOM.findDOMNode(checkbox);
        TestUtils.Simulate.keyDown(node, { which: 32 });

        expect(onChange).not.toHaveBeenCalled();
    });

});
